@@ -7,15 +7,15 @@ use std::collections::HashMap;
 use std::path::Path;
 use std::path::PathBuf;
 
-use super::commit;
 use super::event;
 use super::file_item;
 use super::resolvers;
+use super::sieve;
 
-/// Method how to perform commit of sieved images
+/// Method how to perform sieve of sieved images
 #[derive(PartialEq, Eq, FromPrimitive, ToPrimitive, Clone, Debug, Serialize, Deserialize)]
 #[repr(i32)]
-pub enum CommitMethod {
+pub enum SieveMethod {
     /// Copy the images to be taken over to the target directory
     Copy = 0,
     /// Move the images to be taken over to the target directory
@@ -26,6 +26,19 @@ pub enum CommitMethod {
     Delete,
 }
 
+#[derive(PartialEq, Eq, FromPrimitive, ToPrimitive, Clone, Debug, Serialize, Deserialize)]
+#[repr(i32)]
+pub enum DirectoryNames {
+    /// Directories are named by year and month
+    YearAndMonth = 0,
+    /// Directories are named by year
+    Year,
+    /// Directories are named by year, month and day
+    YearMonthAndDay,
+    /// Directories are named by year and quarter
+    YearAndQuarter,
+}
+
 /// Item list containing all file items and all events
 #[derive(Clone, Debug, Serialize, Deserialize)]
 pub struct ItemList {
@@ -136,16 +149,24 @@ impl ItemList {
         }
     }
 
-    /// Commits an item list taking the take_over flag into account to a new directory.
+    /// Sieves an item list taking the take_over flag into account to a new directory.
     /// The progress is reported by calling a callback function with the file that is currently processed.
-    pub fn commit(
+    pub fn sieve(
         &self,
         path: &Path,
-        commit_method: CommitMethod,
+        sieve_method: SieveMethod,
+        sieve_directory_names: DirectoryNames,
         progress_callback: impl Fn(String),
     ) {
-        let commit_io = commit::FileCommitIO {};
-        commit::commit(self, path, commit_method, &commit_io, progress_callback);
+        let sieve_io = sieve::FileSieveIO {};
+        sieve::sieve(
+            self,
+            path,
+            sieve_method,
+            sieve_directory_names,
+            &sieve_io,
+            progress_callback,
+        );
     }
 
     /// Gets the event which a file item belongs to
@@ -164,20 +185,26 @@ impl ItemList {
 mod tests {
     use super::*;
     use crate::item_sort_list::item_traits::PropertyResolver;
+    use std::cell::RefCell;
+    use std::rc::Rc;
     extern crate base64;
 
-    static mut CALL_COUNT: usize = 0;
+    struct MockResolver {
+        call_count: Rc<RefCell<usize>>,
+    }
 
-    struct MockResolver {}
+    impl MockResolver {
+        pub fn new(call_count: Rc<RefCell<usize>>) -> Self {
+            MockResolver { call_count }
+        }
+    }
 
     impl PropertyResolver for MockResolver {
         fn get_timestamp(&self) -> i64 {
             let return_values: [i64; 6] = [1, 4, 8, 14, 64, 65];
-            #[allow(unsafe_code)]
-            unsafe {
-                CALL_COUNT += 1;
-                return_values[CALL_COUNT - 1]
-            }
+            let call_count = *self.call_count.borrow_mut();
+            self.call_count.replace(call_count + 1);
+            return_values[call_count]
         }
 
         fn get_orientation(&self) -> Option<crate::item_sort_list::Orientation> {
@@ -185,21 +212,15 @@ mod tests {
         }
     }
 
-    fn reset_call_count() {
-        #[allow(unsafe_code)]
-        unsafe {
-            CALL_COUNT = 0;
-        }
-    }
-
     #[test]
     fn find_similar() {
-        reset_call_count();
+        let call_count = Rc::new(RefCell::new(0));
+
         let mut items: Vec<file_item::FileItem> = vec![];
         for _ in 0..6 {
             items.push(file_item::FileItem::new(
                 PathBuf::from(""),
-                Box::new(MockResolver {}),
+                Box::new(MockResolver::new(call_count.clone())),
                 true,
                 "",
             ));
@@ -222,14 +243,15 @@ mod tests {
 
     #[test]
     fn find_similar_hashes() {
-        reset_call_count();
+        let call_count = Rc::new(RefCell::new(0));
+
         let mut items: Vec<file_item::FileItem> = vec![];
         let hashes = ["a", "b", "c", "h", "i", "j"];
         for hash in hashes {
             let encoded = base64::encode(hash).unwrap();
             items.push(file_item::FileItem::new(
                 PathBuf::from(""),
-                Box::new(MockResolver {}),
+                Box::new(MockResolver::new(call_count.clone())),
                 true,
                 &encoded,
             ));
