@@ -1,14 +1,15 @@
-mod commit;
 mod event;
 mod file_item;
 mod item_list;
 mod item_traits;
 mod resolvers;
+mod sieve;
 
 pub use event::parse_date;
 pub use event::Event;
 pub use event::EVENT_DATE_FORMAT;
 pub use file_item::FileItem;
-pub use item_list::CommitMethod;
+pub use item_list::DirectoryNames;
 pub use item_list::ItemList;
+pub use item_list::SieveMethod;
 pub use item_traits::Orientation;
